#include "GeneDetectorConstruction.hh"

#include "GeneDetectorConstructionMessenger.hh"
#include "GeneHe3detSD.hh"
#include <G4Box.hh>
#include <G4Colour.hh>
#include <G4LogicalBorderSurface.hh>
#include <G4LogicalVolume.hh>
#include <G4Material.hh>
#include <G4NistManager.hh>
#include <G4OpticalSurface.hh>
#include <G4PVPlacement.hh>
#include <G4PhysicalConstants.hh>
#include <G4RotationMatrix.hh>
#include <G4SDManager.hh>
#include <G4SubtractionSolid.hh>
#include <G4SystemOfUnits.hh>
#include <G4ThreeVector.hh>
#include <G4Tubs.hh>
#include <G4VisAttributes.hh>
#include <globals.hh>

GeneDetectorConstruction::GeneDetectorConstruction() {
    Zoffset = 0;
    Zactiveshift = 0;
    PE_Bfraction = 0.054;
    SReaction = "C13an";
    pMessenger = new GeneDetectorConstructionMessenger(this);
}

GeneDetectorConstruction::~GeneDetectorConstruction() { delete pMessenger; }

G4VPhysicalVolume* GeneDetectorConstruction::Construct() {
    // ********** Materials **********
    G4NistManager* manager = G4NistManager::Instance();
    G4double z, n, a, density, temperature, pressure, fractionmass;
    G4String name, symbol;
    G4int ncomponents;

    // Hydrogen 氢
    G4Element* ele_H = manager->FindOrBuildElement("H");
    // Carbon 碳
    G4Element* ele_C = manager->FindOrBuildElement("C");
    // Boron 硼
    G4Material* mat_B = manager->FindOrBuildMaterial("G4_B");
    // Copper 铜
    G4Material* mat_Cu = manager->FindOrBuildMaterial("G4_Cu");
    // Gold 金
    G4Material* mat_Au = manager->FindOrBuildMaterial("G4_Au");
    // Aluminum 铝
    G4Material* mat_Al = manager->FindOrBuildMaterial("G4_Al");
    // Tantalum 钽
    G4Material* mat_Ta = manager->FindOrBuildMaterial("G4_Ta");
    // Stainless stell 不锈钢
    G4Material* mat_stell = manager->FindOrBuildMaterial("G4_STAINLESS-STEEL");
    // Water 水
    G4Material* mat_Water = manager->FindOrBuildMaterial("G4_WATER");
    // Air 空气
    G4Material* mat_Air = manager->FindOrBuildMaterial("G4_AIR");
    // Polyethylene 聚乙烯
    G4Material* mat_PE = manager->FindOrBuildMaterial("G4_POLYETHYLENE");

    // shielding 屏蔽层 PE(B) 含硼聚乙烯
    density = 0.938 * g / cm3;
    G4Material* shield_PE = new G4Material("PE_B", density, ncomponents = 2);
    shield_PE->AddMaterial(mat_PE, fractionmass = 93 * perCent);
    shield_PE->AddMaterial(mat_B, fractionmass = 7 * perCent);

    // moderator 慢化剂 PE(B) 含硼聚乙烯
    G4Material* moderator_PE = new G4Material("PE_B_impurity", density, ncomponents = 2);
    moderator_PE->AddMaterial(mat_PE, fractionmass = (100.0 - PE_Bfraction) * perCent);
    moderator_PE->AddMaterial(mat_B, fractionmass = PE_Bfraction * perCent);

    // Vacuum 真空
    temperature = 77 * kelvin;
    pressure = 1.0e-5 * pascal;
    density = 1.29 * kg / m3 * pressure / atmosphere;
    G4Material* Vacuum = new G4Material("Vacuum", density, ncomponents = 1, kStateGas, temperature, pressure);
    Vacuum->AddMaterial(mat_Air, 100.0 * perCent);

    // He3 gas He3气体
    G4Isotope* iso_He3 = new G4Isotope(name = "He3", z = 2.0, n = 3.0, a = 3.016 * g / mole);
    G4Element* ele_He = new G4Element(name = "iso_He3", symbol = "He3", ncomponents = 1);
    ele_He->AddIsotope(iso_He3, 100.0 * perCent);

    // Work gas 工作气体
    temperature = 300. * kelvin;
    pressure = 10. * atmosphere;
    density = 1.22685 * mg / cm3;  // 0.49074(3He, 4atm) Calculate by LISE++
    G4Material* work_gas = new G4Material("Workgas", density, ncomponents = 1, kStateGas, temperature, pressure);
    work_gas->AddElement(ele_He, 100. * perCent);

    // C13 Target C13靶
    G4Isotope* iso_C12 = new G4Isotope(name = "C12", z = 6.0, n = 12.0, a = 12.0 * g / mole);
    G4Isotope* iso_C13 = new G4Isotope(name = "C13", z = 6.0, n = 13.0, a = 13.003 * g / mole);
    G4Element* ele_C13 = new G4Element(name = "Carbon", symbol = "C13", ncomponents = 2);
    ele_C13->AddIsotope(iso_C12, 1.0 * perCent);
    ele_C13->AddIsotope(iso_C13, 99.0 * perCent);
    G4Material* target_C = new G4Material("targetC", density = 2.3 * g / cm3, ncomponents = 1);
    target_C->AddElement(ele_C13, 100 * perCent);

    // ********** Sciintillator **********
    // Plastic scintillator 塑料闪烁体 EJ-200
    density = 1.023 * g / cm3;
    G4Material* ej_200 = new G4Material("ej_200", density, ncomponents = 2);
    ej_200->AddElement(ele_H, 8.5 * perCent);
    ej_200->AddElement(ele_C, 91.5 * perCent);

    // Material properties table 物质光学属性表
    G4MaterialPropertiesTable* ej_200_MPT = new G4MaterialPropertiesTable();
    // Optical properties 光学属性
    const G4int nEntries = 21;
    G4double PhotonEnergy[nEntries] = {2.411 * eV, 2.483 * eV, 2.533 * eV, 2.586 * eV, 2.628 * eV, 2.666 * eV,
                                       2.699 * eV, 2.779 * eV, 2.821 * eV, 2.865 * eV, 2.903 * eV, 2.921 * eV,
                                       2.936 * eV, 2.946 * eV, 2.964 * eV, 3.008 * eV, 3.022 * eV, 3.039 * eV,
                                       3.079 * eV, 3.104 * eV, 3.143 * eV};  // 光子能量 395 nm -> 515 nm
    G4double AbsorptionLength[nEntries] = {380. * cm, 380. * cm, 380. * cm, 380. * cm, 380. * cm, 380. * cm,
                                           380. * cm, 380. * cm, 380. * cm, 380. * cm, 380. * cm, 380. * cm,
                                           380. * cm, 380. * cm, 380. * cm, 380. * cm, 380. * cm, 380. * cm,
                                           380. * cm, 380. * cm, 380. * cm};  // 吸收长度
    G4double RefractiveIndex[nEntries] = {1.58, 1.58, 1.58, 1.58, 1.58, 1.58, 1.58, 1.58, 1.58, 1.58, 1.58,
                                          1.58, 1.58, 1.58, 1.58, 1.58, 1.58, 1.58, 1.58, 1.58, 1.58};  // 折射率
    G4double FastComponent[nEntries] = {0.00000, 0.05885, 0.10053, 0.17197, 0.26426, 0.37440, 0.43394,
                                        0.60064, 0.74651, 0.89833, 0.98168, 1.00000, 0.97870, 0.93703,
                                        0.79712, 0.40119, 0.30444, 0.19876, 0.03801, 0.00926, 0.00000};  // 快成分能谱
    ej_200_MPT->AddProperty("ABSLENGTH", PhotonEnergy, AbsorptionLength, nEntries);
    ej_200_MPT->AddProperty("RINDEX", PhotonEnergy, RefractiveIndex, nEntries);
    ej_200_MPT->AddProperty("FASTCOMPONENT", PhotonEnergy, FastComponent, nEntries);

    // Intrinsic properties 本征属性
    ej_200_MPT->AddConstProperty("RESOLUTIONSCALE", 10.0);                // 本征分辨率
    ej_200_MPT->AddConstProperty("SCINTILLATIONYIELD", 10000. / MeV);     // 本征光产额
    ej_200_MPT->AddConstProperty("YIELDRATIO", 1.0);                      // 快慢响应之比
    ej_200_MPT->AddConstProperty("FASTSCINTILLATIONRISETIME", 0.9 * ns);  // 快成分上升时间
    ej_200_MPT->AddConstProperty("FASTTIMECONSTANT", 2.1 * ns);           // 快成分衰减时间
    ej_200->SetMaterialPropertiesTable(ej_200_MPT);

    // ********** Boundary Optical Properties **********
    G4OpticalSurface* ej_200_OpticalSurface = new G4OpticalSurface("ej_200_OpticalSurface");
    ej_200_OpticalSurface->SetType(dielectric_metal);
    ej_200_OpticalSurface->SetFinish(polished);
    ej_200_OpticalSurface->SetModel(glisur);

    // ********** Geometry **********
    // world
    G4double world_xyz = 2. * m;
    G4Box* solidWorld = new G4Box("world", 0.5 * world_xyz, 0.5 * world_xyz, 0.5 * world_xyz);
    logicalWorld = new G4LogicalVolume(solidWorld, mat_Air, "world", 0, 0, 0);
    physicalWorld = new G4PVPlacement(0, G4ThreeVector(), "world", logicalWorld,
                                      NULL,   // mother volume
                                      false,  // no boolean operation
                                      0);     // copy number
    logicalWorld->SetVisAttributes(G4VisAttributes::GetInvisible);

    // shielding PE
    G4double BPE_xyz = 60.0 * cm;
    G4Box* BPE_box = new G4Box("BPE_box", 0.5 * BPE_xyz, 0.5 * BPE_xyz, 0.5 * BPE_xyz);
    // moderator PE
    G4double PE_xyz = 50.0 * cm;
    G4Box* PE_box = new G4Box("PE_box", 0.5 * PE_xyz, 0.5 * PE_xyz, 0.5 * PE_xyz);

    // Beam hole
    G4double rMin0 = 0.;
    G4double rMax0 = 5.0 * cm;
    G4Tubs* PE_hole = new G4Tubs("PE_hole", rMin0, rMax0, 0.5 * BPE_xyz, 0, 360. * deg);

    // He3 Tube holes
    G4double active_len = 30 * cm;  // length
    G4double dead_len = 2 * cm;
    G4double tube_r = 2.54 / 2 * cm;  // radius
    G4double tube_t = 0.5 * mm;       // thickness

    G4double gas_region_len = active_len + 2.0 * dead_len;  // 34cm
    G4double gas_region_r = tube_r - tube_t;                // active region radius and dead region radius is the same
    G4double tube_len = gas_region_len + 2 * tube_t;        // 32.1cm

    G4double tube_hole_len = 0.5 * BPE_xyz + 0.5 * active_len + dead_len + 1 * cm;  // 48cm
    G4double tube_hole_r = 0.5 * 2.745 * cm;
    G4double tube_hole_z = -0.5 * BPE_xyz + 0.5 * tube_hole_len;
    G4Tubs* He3tube_Hole = new G4Tubs("He3tube_Hole", 0, tube_hole_r, 0.5 * tube_hole_len, 0, 360. * deg);

    G4Tubs* He3tub = new G4Tubs("He3tub", 0, tube_r, 0.5 * tube_len, 0, 360. * deg);
    G4Tubs* He3gas = new G4Tubs("He3gas", 0, gas_region_r, 0.5 * gas_region_len, 0, 360. * deg);
    G4Tubs* He3active = new G4Tubs("He3active", 0, gas_region_r, 0.5 * active_len, 0, 360. * deg);
    G4SubtractionSolid* He3tub_sub = new G4SubtractionSolid("He3tub_sub", He3tub, He3gas);
    G4RotationMatrix rot0;
    G4SubtractionSolid* He3gas_sub =
        new G4SubtractionSolid("He3gas_sub", He3gas, He3active, G4Transform3D(rot0, G4ThreeVector(0, 0, Zactiveshift)));

    G4LogicalVolume* He3tub_log = new G4LogicalVolume(He3tub_sub, mat_stell, "He3tub_log");
    G4LogicalVolume* He3gas_log = new G4LogicalVolume(He3gas_sub, work_gas, "He3gas_log");
    G4LogicalVolume* He3active_log = new G4LogicalVolume(He3active, work_gas, "He3active_log");

    // G4VisAttributes* He3gas_vis= new G4VisAttributes(G4Colour(1.0,1.0,0.0,0.1));
    // He3gas_vis->SetForceSolid(true);
    // He3gas_vis->SetForceWireframe(true);
    // He3gas_log->SetVisAttributes(He3gas_vis);
    G4VisAttributes* He3active_vis = new G4VisAttributes(G4Colour(1.0, 0.0, 0.0));
    He3active_vis->SetForceSolid(true);
    He3active_log->SetVisAttributes(He3active_vis);
    // He3gas_log -> SetVisAttributes(G4VisAttributes::GetInvisible);
    He3tub_log->SetVisAttributes(G4VisAttributes::GetInvisible);
    // He3active_log -> SetVisAttributes(G4VisAttributes::GetInvisible);

    G4SubtractionSolid* solid_PE = new G4SubtractionSolid("solid_PE", PE_box, PE_hole);
    G4SubtractionSolid* sub_BPE = new G4SubtractionSolid("sub_BPE", BPE_box, PE_box);
    G4SubtractionSolid* solid_BPE = new G4SubtractionSolid("solid_BPE", sub_BPE, PE_hole);

    char tmp[50];
    G4double posx, posy;
    for (G4int i = 0; i < 12; i++) {
        posx = 8.5 * cm * std::cos(i * 30.0 * deg);
        posy = 8.5 * cm * std::sin(i * 30.0 * deg);
        sprintf(tmp, "solid_PE%d", i);
        solid_PE = new G4SubtractionSolid(tmp, solid_PE, He3tube_Hole,
                                          G4Transform3D(rot0, G4ThreeVector(posx, posy, tube_hole_z)));
        sprintf(tmp, "solid_BPE%d", i);
        solid_BPE = new G4SubtractionSolid(tmp, solid_BPE, He3tube_Hole,
                                           G4Transform3D(rot0, G4ThreeVector(posx, posy, tube_hole_z)));
        posx = 13.0 * cm * std::cos(i * 30.0 * deg + 15.0 * deg);
        posy = 13.0 * cm * std::sin(i * 30.0 * deg + 15.0 * deg);
        sprintf(tmp, "solid_PE%d", i + 12);
        solid_PE = new G4SubtractionSolid(tmp, solid_PE, He3tube_Hole,
                                          G4Transform3D(rot0, G4ThreeVector(posx, posy, tube_hole_z)));
        sprintf(tmp, "solid_BPE%d", i + 12);
        solid_BPE = new G4SubtractionSolid(tmp, solid_BPE, He3tube_Hole,
                                           G4Transform3D(rot0, G4ThreeVector(posx, posy, tube_hole_z)));
    }
    // z offset for position measurement
    G4LogicalVolume* PE_log = new G4LogicalVolume(solid_PE, moderator_PE, "logic_PE");
    new G4PVPlacement(0, G4ThreeVector(0, 0, Zoffset), "PE_phys", PE_log, physicalWorld, false, 1, true);

    G4LogicalVolume* BPE_log = new G4LogicalVolume(solid_BPE, shield_PE, "logic_BPE");
    new G4PVPlacement(0, G4ThreeVector(0, 0, Zoffset), "BPE_phys", BPE_log, physicalWorld, false, 2, true);

    G4VisAttributes* PE_vis = new G4VisAttributes(G4Colour(0.0, 0.8, 0.8, 0.5));
    PE_vis->SetForceSolid(true);
    // PE_vis->SetForceWireframe(true);
    PE_log->SetVisAttributes(PE_vis);

    // G4VisAttributes* BPE_vis= new G4VisAttributes(G4Colour(1.0,1.0,1.0,0.1));
    // BPE_vis->SetForceSolid(true);
    // BPE_vis->SetForceWireframe(true);
    // BPE_log->SetVisAttributes(BPE_vis);

    // PE_log -> SetVisAttributes(G4VisAttributes::GetInvisible);

    G4VisAttributes* BPE_vis = new G4VisAttributes(G4Colour(0.1, 0.3, 1, 0.8));
    BPE_log->SetVisAttributes(BPE_vis);

    G4double posz = tube_hole_len - 0.5 * BPE_xyz - 0.5 * tube_len;
    for (G4int i = 0; i < 12; i++) {
        posx = 8.5 * cm * std::cos(i * 30.0 * deg);
        posy = 8.5 * cm * std::sin(i * 30.0 * deg);
        sprintf(tmp, "He3tub_phys%d", i);
        new G4PVPlacement(0, G4ThreeVector(posx, posy, posz + Zoffset), tmp, He3tub_log, physicalWorld, false, 100 + i,
                          true);
        sprintf(tmp, "He3gas_phys%d", i);
        new G4PVPlacement(0, G4ThreeVector(posx, posy, posz + Zoffset), tmp, He3gas_log, physicalWorld, false, 200 + i,
                          true);
        sprintf(tmp, "He3active_phys%d", i);
        new G4PVPlacement(0, G4ThreeVector(posx, posy, posz + Zoffset + Zactiveshift), tmp, He3active_log,
                          physicalWorld, false, 300 + i, true);
        posx = 13.0 * cm * std::cos(i * 30.0 * deg + 15.0 * deg);
        posy = 13.0 * cm * std::sin(i * 30.0 * deg + 15.0 * deg);
        sprintf(tmp, "He3tub_phys%d", i + 12);
        new G4PVPlacement(0, G4ThreeVector(posx, posy, posz + Zoffset), tmp, He3tub_log, physicalWorld, false,
                          100 + i + 12, true);
        sprintf(tmp, "He3gas_phys%d", i + 12);
        new G4PVPlacement(0, G4ThreeVector(posx, posy, posz + Zoffset), tmp, He3gas_log, physicalWorld, false,
                          200 + i + 12, true);
        sprintf(tmp, "He3active_phys%d", i + 12);
        new G4PVPlacement(0, G4ThreeVector(posx, posy, posz + Zoffset + Zactiveshift), tmp, He3active_log,
                          physicalWorld, false, 300 + i + 12, true);
    }

    G4VisAttributes* C13_vis = new G4VisAttributes(G4Colour(1.0, 1.0, 1.0, 1.0));
    C13_vis->SetForceSolid(true);
    G4VisAttributes* CuSub_vis = new G4VisAttributes(G4Colour(0.1, 0.5, 0.7, 1.0));
    CuSub_vis->SetForceSolid(true);
    G4VisAttributes* tube_vis = new G4VisAttributes(G4Colour(0.8, 0.8, 0.8, 1.0));
    tube_vis->SetForceSolid(true);
    G4VisAttributes* water_vis = new G4VisAttributes(G4Colour(0.0, 0.0, 0.1, 1.0));
    water_vis->SetForceSolid(true);

    if (SReaction == "C13an") {
        G4double C13thickness = 2.0 * mm;
        G4Tubs* C13tub = new G4Tubs("C13tub", 0, 20. * mm, 0.5 * C13thickness, 0, 360. * deg);

        G4LogicalVolume* C13log = new G4LogicalVolume(C13tub, target_C, "C13log");
        new G4PVPlacement(0, G4ThreeVector(), "C13phys", C13log, physicalWorld, false, 3, true);
        C13log->SetVisAttributes(C13_vis);
        // C13log -> SetVisAttributes(G4VisAttributes::GetInvisible);

        // Cu_Substrate
        G4double Cu0thickness = 1.5 * mm;
        G4double Cu0z = 0.5 * (C13thickness + Cu0thickness);
        G4Tubs* CuSubstrate_tub0 =
            new G4Tubs("CuSubstrate_tub0", 18. * mm, 25. * mm, 0.5 * Cu0thickness, 0, 360. * deg);
        G4LogicalVolume* CuSubstrate_log0 = new G4LogicalVolume(CuSubstrate_tub0, mat_Cu, "CuSubstrate_log0");
        new G4PVPlacement(0, G4ThreeVector(0, 0, Cu0z), "CuSubstrate_phys0", CuSubstrate_log0, physicalWorld, false, 4,
                          true);

        G4double Cu1thickness = C13thickness;
        G4double Cu1z = 0.0 * mm;
        G4Tubs* CuSubstrate_tub1 =
            new G4Tubs("CuSubstrate_tub1", 20. * mm, 24.25 * mm, 0.5 * Cu1thickness, 0, 360. * deg);
        G4LogicalVolume* CuSubstrate_log1 = new G4LogicalVolume(CuSubstrate_tub1, mat_Cu, "CuSubstrate_log1");
        new G4PVPlacement(0, G4ThreeVector(0, 0, Cu1z), "CuSubstrate_phys1", CuSubstrate_log1, physicalWorld, false, 5,
                          true);

        G4double Cu2thickness = 3.5 * mm;
        G4double Cu2z = -0.5 * (C13thickness + Cu2thickness);
        G4Tubs* CuSubstrate_tub2 = new G4Tubs("CuSubstrate_tub2", 0, 24.25 * mm, 0.5 * Cu2thickness, 0, 360. * deg);
        G4LogicalVolume* CuSubstrate_log2 = new G4LogicalVolume(CuSubstrate_tub2, mat_Cu, "CuSubstrate_log2");
        new G4PVPlacement(0, G4ThreeVector(0, 0, Cu2z), "CuSubstrate_phys2", CuSubstrate_log2, physicalWorld, false, 6,
                          true);

        G4double Cu3thickness = 2.0 * mm;
        G4double Cu3z = Cu2z - 0.5 * Cu2thickness - 0.5 * Cu3thickness;
        G4Tubs* CuSubstrate_tub3 = new G4Tubs("CuSubstrate_tub3", 0, 29. * mm, 0.5 * Cu3thickness, 0, 360. * deg);
        G4LogicalVolume* CuSubstrate_log3 = new G4LogicalVolume(CuSubstrate_tub3, mat_Cu, "CuSubstrate_log3");
        new G4PVPlacement(0, G4ThreeVector(0, 0, Cu3z), "CuSubstrate_phys3", CuSubstrate_log3, physicalWorld, false, 7,
                          true);

        CuSubstrate_log0->SetVisAttributes(CuSub_vis);
        CuSubstrate_log1->SetVisAttributes(CuSub_vis);
        CuSubstrate_log2->SetVisAttributes(CuSub_vis);
        CuSubstrate_log3->SetVisAttributes(CuSub_vis);

        // Target flange
        G4double flange0_thickness = 9.0 * mm;
        G4double flange0z = Cu2z - 0.5 * Cu2thickness + 0.5 * flange0_thickness;
        G4Tubs* flange_tub0 = new G4Tubs("flange_tub0", 25. * mm, 42.33 * mm, 0.5 * flange0_thickness, 0, 360. * deg);
        G4LogicalVolume* flange_log0 = new G4LogicalVolume(flange_tub0, mat_stell, "flange_log0");
        new G4PVPlacement(0, G4ThreeVector(0, 0, flange0z), "flange_phys0", flange_log0, physicalWorld, false, 8, true);

        G4double flange1_thickness = 7.0 * mm;
        G4double flange1z = flange0z - 0.5 * (flange0_thickness + flange1_thickness);
        G4Tubs* flange_tub1 = new G4Tubs("flange_tub1", 29. * mm, 44.45 * mm, 0.5 * flange1_thickness, 0, 360. * deg);
        G4LogicalVolume* flange_log1 = new G4LogicalVolume(flange_tub1, mat_stell, "flange_log1");
        new G4PVPlacement(0, G4ThreeVector(0, 0, flange1z), "flange_phys1", flange_log1, physicalWorld, false, 9, true);

        // BeamLine_stainlesss
        G4double BeampipeLength = 60.0 * cm;
        G4double Beampipez = flange0z - 0.5 * flange0_thickness + 0.5 * BeampipeLength;
        G4Tubs* Beampipe_tub =
            new G4Tubs("Beampipe_tub", 42.33 * mm, 44.45 * mm, 0.5 * BeampipeLength, 0. * deg, 360. * deg);
        G4LogicalVolume* Beampipe_log = new G4LogicalVolume(Beampipe_tub, mat_stell, "Beampipe_log");
        new G4PVPlacement(0, G4ThreeVector(0, 0, Beampipez), "Beampipe_phys", Beampipe_log, physicalWorld, false, 10,
                          true);

        flange_log0->SetVisAttributes(tube_vis);
        flange_log1->SetVisAttributes(tube_vis);
        Beampipe_log->SetVisAttributes(tube_vis);

        // Cold trap
        G4double CuTrapLength = 50.0 * cm;
        G4double CuTrapz = 0.5 * CuTrapLength + 12.0 * mm + 60.0 * mm;
        G4Tubs* CuTrap_tub = new G4Tubs("CuTrap_tub", 28.0 * mm, 30.0 * mm, 0.5 * CuTrapLength, 0, 360. * deg);
        G4LogicalVolume* CuTrap_log = new G4LogicalVolume(CuTrap_tub, mat_Cu, "CuTrap_log");
        new G4PVPlacement(0, G4ThreeVector(0, 0, CuTrapz), "CuTrap_phys", CuTrap_log, physicalWorld, false, 11, true);

        CuTrap_log->SetVisAttributes(CuSub_vis);

        // Suppressor
        G4double CuSupLength = 1.0 * mm;
        G4double CuSupz = 0.5 * CuSupLength + 8.0 * mm + 60.0 * mm;
        G4Tubs* CuSup_tub = new G4Tubs("CuSup_tub", 15.0 * mm, 30.0 * mm, 0.5 * CuSupLength, 0, 360. * deg);
        G4LogicalVolume* CuSup_log = new G4LogicalVolume(CuSup_tub, mat_Cu, "CuSup_log");
        new G4PVPlacement(0, G4ThreeVector(0, 0, CuSupz), "CuSup_phys", CuSup_log, physicalWorld, false, 12, true);

        // Collimator
        G4double AuCollLength = 3.0 * um;
        G4double AuCollz = -0.5 * AuCollLength + 12.0 * mm + 60.0 * mm;
        G4Tubs* AuColl_tub = new G4Tubs("AuColl_tub", 5.0 * mm, 28.0 * mm, 0.5 * AuCollLength, 0, 360. * deg);
        G4LogicalVolume* AuColl_log = new G4LogicalVolume(AuColl_tub, mat_Au, "AuColl_log");
        new G4PVPlacement(0, G4ThreeVector(0, 0, AuCollz), "AuColl_phys", AuColl_log, physicalWorld, false, 13, true);

        G4double CuColl0Length = 2.0 * mm;
        G4double CuColl0z = 0.5 * CuColl0Length + 12.0 * mm + 60.0 * mm;
        G4Tubs* CuColl_tub0 = new G4Tubs("CuColl_tub0", 5.0 * mm, 28.0 * mm, 0.5 * CuColl0Length, 0, 360. * deg);
        G4LogicalVolume* CuColl_log0 = new G4LogicalVolume(CuColl_tub0, mat_Cu, "CuColl_log0");
        new G4PVPlacement(0, G4ThreeVector(0, 0, CuColl0z), "CuColl_phys0", CuColl_log0, physicalWorld, false, 14,
                          true);

        G4double CuColl1Length = 3.0 * mm;
        G4double CuColl1z = CuColl0z + 0.5 * CuColl0Length + 0.5 * CuColl1Length;
        G4Tubs* CuColl_tub1 = new G4Tubs("CuColl_tub1", 25.0 * mm, 28.0 * mm, 0.5 * CuColl1Length, 0, 360. * deg);
        G4LogicalVolume* CuColl_log1 = new G4LogicalVolume(CuColl_tub1, mat_Cu, "CuColl_log1");
        new G4PVPlacement(0, G4ThreeVector(0, 0, CuColl1z), "CuColl_phys1", CuColl_log1, physicalWorld, false, 15,
                          true);

        G4double VacuumLength = 70.0 * cm;
        G4double Vacuumz = 0.5 * VacuumLength + Cu2z - 0.5 * Cu2thickness;
        G4Tubs* Vacuum_tub = new G4Tubs("Vacuum_tub", 0, 42.33 * mm, 0.5 * VacuumLength, 0, 360. * deg);
        G4double VacuumTflangez = -0.5 * VacuumLength + 0.5 * flange0_thickness;
        G4SubtractionSolid* Vacuum_sub =
            new G4SubtractionSolid("VacuumTflange_sub", Vacuum_tub, flange_tub0,
                                   G4Transform3D(rot0, G4ThreeVector(0, 0, VacuumTflangez)));  // Target flange0
        G4double VacuumCu2z = -0.5 * VacuumLength + 0.5 * Cu2thickness;
        Vacuum_sub = new G4SubtractionSolid("Vacuum_sub2", Vacuum_sub, CuSubstrate_tub2,
                                            G4Transform3D(rot0, G4ThreeVector(0, 0, VacuumCu2z)));  // Cu2
        G4double VacuumCu1z = VacuumCu2z + 0.5 * Cu2thickness + 0.5 * Cu1thickness;
        Vacuum_sub = new G4SubtractionSolid("Vacuum_sub1", Vacuum_sub, CuSubstrate_tub1,
                                            G4Transform3D(rot0, G4ThreeVector(0, 0, VacuumCu1z)));  // Cu1
        G4double VacuumCu0z = VacuumCu1z + 0.5 * Cu1thickness + 0.5 * Cu0thickness;
        Vacuum_sub = new G4SubtractionSolid("Vacuum_sub0", Vacuum_sub, CuSubstrate_tub0,
                                            G4Transform3D(rot0, G4ThreeVector(0, 0, VacuumCu0z)));  // Cu0
        G4double VacuumC13z = VacuumCu1z;
        Vacuum_sub = new G4SubtractionSolid("VacuumC13_sub", Vacuum_sub, C13tub,
                                            G4Transform3D(rot0, G4ThreeVector(0, 0, VacuumC13z)));  // C13 target
        G4double VacuumCuTrapz = VacuumC13z + 12.0 * mm + 0.5 * CuTrapLength + 60.0 * mm;
        Vacuum_sub = new G4SubtractionSolid("VacuumCuTrap_sub", Vacuum_sub, CuTrap_tub,
                                            G4Transform3D(rot0, G4ThreeVector(0, 0, VacuumCuTrapz)));  // Cu Trap
        G4double VacuumCuSupz = VacuumC13z + 8.0 * mm + 0.5 * CuSupLength + 60.0 * mm;
        Vacuum_sub = new G4SubtractionSolid("VacuumCuSup_sub", Vacuum_sub, CuSup_tub,
                                            G4Transform3D(rot0, G4ThreeVector(0, 0, VacuumCuSupz)));  // Cu Suppressor
        G4double VacuumAuCollz = VacuumC13z + 12.0 * mm - 0.5 * AuCollLength + 60.0 * mm;
        Vacuum_sub = new G4SubtractionSolid("VacuumAuColl_sub", Vacuum_sub, AuColl_tub,
                                            G4Transform3D(rot0, G4ThreeVector(0, 0, VacuumAuCollz)));  // Au Collimator
        G4double VacuumCuColl0z = VacuumC13z + 12.0 * mm + 0.5 * CuColl0Length + 60.0 * mm;
        Vacuum_sub =
            new G4SubtractionSolid("VacuumCuColl_sub0", Vacuum_sub, CuColl_tub0,
                                   G4Transform3D(rot0, G4ThreeVector(0, 0, VacuumCuColl0z)));  // Cu Collimator0
        G4double VacuumCuColl1z = VacuumCuColl0z + 0.5 * CuColl0Length + 0.5 * CuColl1Length;
        Vacuum_sub =
            new G4SubtractionSolid("VacuumCuColl_sub1", Vacuum_sub, CuColl_tub1,
                                   G4Transform3D(rot0, G4ThreeVector(0, 0, VacuumCuColl1z)));  // Cu Collimator1
        G4LogicalVolume* Vacuum_log = new G4LogicalVolume(Vacuum_sub, Vacuum, "Vacuum_log");
        new G4PVPlacement(0, G4ThreeVector(0, 0, Vacuumz), "Vacuum_phys", Vacuum_log, physicalWorld, false, 16, true);

        // Water_cold
        G4double WaterBack_thickness = 1.0 * mm;
        G4double WaterBackz = Cu3z - 0.5 * Cu3thickness - 0.5 * WaterBack_thickness;
        G4Tubs* WaterBack_tub =
            new G4Tubs("WaterBack_tub", 0, 29.0 * mm, 0.5 * WaterBack_thickness, 0. * deg, 360. * deg);
        G4LogicalVolume* WaterBack_log = new G4LogicalVolume(WaterBack_tub, mat_Water, "WaterBack_log");
        new G4PVPlacement(0, G4ThreeVector(0, 0, WaterBackz), "WaterBack_phys", WaterBack_log, physicalWorld, false, 17,
                          true);
        WaterBack_log->SetVisAttributes(water_vis);

        // Al Substrate
        G4double Al0thickness = 4.0 * mm;
        G4double Al0z = WaterBackz - 0.5 * WaterBack_thickness - 0.5 * Al0thickness;
        G4Tubs* AlSubstrate_tub0 =
            new G4Tubs("AlSubstrate_tub0", 0, 29.0 * mm, 0.5 * Al0thickness, 0. * deg, 360. * deg);
        G4LogicalVolume* AlSubstrate_log0 = new G4LogicalVolume(AlSubstrate_tub0, mat_Al, "AlSubstrate_log0");
        G4PVPlacement* Al0phys = new G4PVPlacement(0, G4ThreeVector(0, 0, Al0z), "AlSubstrate_phys0", AlSubstrate_log0,
                                                   physicalWorld, false, 18, true);
        AlSubstrate_log0->SetVisAttributes(tube_vis);

        G4double Al1thickness = 4.0 * mm;
        G4double Al1z = Al0z - 0.5 * Al0thickness - 0.5 * Al1thickness;
        G4Tubs* AlSubstrate_tub1 =
            new G4Tubs("AlSubstrate_tub1", 0, 44.45 * mm, 0.5 * Al1thickness, 0. * deg, 360. * deg);
        G4LogicalVolume* AlSubstrate_log1 = new G4LogicalVolume(AlSubstrate_tub1, mat_Al, "AlSubstrate_log1");
        G4PVPlacement* Al1phys = new G4PVPlacement(0, G4ThreeVector(0, 0, Al1z), "AlSubstrate_phys1", AlSubstrate_log1,
                                                   physicalWorld, false, 19, true);
        AlSubstrate_log1->SetVisAttributes(tube_vis);

        // Waterpipe
        G4double Waterpipe0_thickness = 4.0 * mm;
        G4Tubs* Waterpipe_tub0 =
            new G4Tubs("Waterpipe_tub0", 0, 2.5 * mm, 0.5 * Waterpipe0_thickness, 0. * deg, 360. * deg);
        G4LogicalVolume* Waterpipe_log0 = new G4LogicalVolume(Waterpipe_tub0, mat_Water, "Waterpipe_log0");
        new G4PVPlacement(0, G4ThreeVector(0, 21.5 * mm, 0), "Waterpipe_phys00", Waterpipe_log0, Al0phys, false, 20,
                          true);
        new G4PVPlacement(0, G4ThreeVector(0, -21.5 * mm, 0), "Waterpipe_phys01", Waterpipe_log0, Al0phys, false, 21,
                          true);
        new G4PVPlacement(0, G4ThreeVector(0, 21.5 * mm, 0), "Waterpipe_phys02", Waterpipe_log0, Al1phys, false, 22,
                          true);
        new G4PVPlacement(0, G4ThreeVector(0, -21.5 * mm, 0), "Waterpipe_phys03", Waterpipe_log0, Al1phys, false, 23,
                          true);

        G4double Waterpipe1_thickness = BPE_xyz / 2.0 + 10.0 * cm;
        G4double Waterpipe1z = Al1z - 0.5 * Al1thickness - 0.5 * Waterpipe1_thickness;
        G4Tubs* Waterpipe_tub1 =
            new G4Tubs("Waterpipe_tub1", 0, 2.5 * mm, 0.5 * Waterpipe1_thickness, 0. * deg, 360. * deg);
        G4LogicalVolume* Waterpipe_log1 = new G4LogicalVolume(Waterpipe_tub1, mat_Water, "Waterpipe_log1");
        new G4PVPlacement(0, G4ThreeVector(0, 21.5 * mm, Waterpipe1z), "Waterpipe_phys1", Waterpipe_log1, physicalWorld,
                          false, 24, true);
        new G4PVPlacement(0, G4ThreeVector(0, -21.5 * mm, Waterpipe1z), "Waterpipe_phys2", Waterpipe_log1,
                          physicalWorld, false, 25, true);
        Waterpipe_log0->SetVisAttributes(water_vis);
        Waterpipe_log1->SetVisAttributes(water_vis);

    } else if (SReaction == "V51pn") {
        // V51 on Ta target
        G4double Tathickness = 1.0 * mm;
        G4Tubs* Tatub = new G4Tubs("Tatub", 0, 15. * mm, 0.5 * Tathickness, 0, 360. * deg);

        G4LogicalVolume* Talog = new G4LogicalVolume(Tatub, mat_Ta, "Talog");
        new G4PVPlacement(0, G4ThreeVector(), "Taphys", Talog, physicalWorld, false, 3, true);
        Talog->SetVisAttributes(C13_vis);

        // Cu_Substrate
        G4double Cu0thickness = 8.0 * mm;
        G4double Cu0z = 0.5 * (Tathickness + Cu0thickness);
        G4Tubs* CuSubstrate_tub0 =
            new G4Tubs("CuSubstrate_tub0", 11. * mm, 29. * mm, 0.5 * Cu0thickness, 0, 360. * deg);
        G4LogicalVolume* CuSubstrate_log0 = new G4LogicalVolume(CuSubstrate_tub0, mat_Cu, "CuSubstrate_log0");
        new G4PVPlacement(0, G4ThreeVector(0, 0, Cu0z), "CuSubstrate_phys1", CuSubstrate_log0, physicalWorld, false, 4,
                          true);

        CuSubstrate_log0->SetVisAttributes(CuSub_vis);

        // Target flange
        G4double flange0_thickness = 9.0 * mm;
        G4double flange0z = Cu0z + 0.5 * Cu0thickness + 0.5 * flange0_thickness;
        G4Tubs* flange_tub0 = new G4Tubs("flange_tub0", 25. * mm, 42.33 * mm, 0.5 * flange0_thickness, 0, 360. * deg);
        G4LogicalVolume* flange_log0 = new G4LogicalVolume(flange_tub0, mat_stell, "flange_log0");
        new G4PVPlacement(0, G4ThreeVector(0, 0, flange0z), "flange_phys0", flange_log0, physicalWorld, false, 8, true);

        G4double flange1_thickness = 7.0 * mm;
        G4double flange1z = flange0z - 0.5 * (flange0_thickness + flange1_thickness);
        G4Tubs* flange_tub1 = new G4Tubs("flange_tub1", 29. * mm, 44.45 * mm, 0.5 * flange1_thickness, 0, 360. * deg);
        G4LogicalVolume* flange_log1 = new G4LogicalVolume(flange_tub1, mat_stell, "flange_log1");
        new G4PVPlacement(0, G4ThreeVector(0, 0, flange1z), "flange_phys1", flange_log1, physicalWorld, false, 9, true);

        // BeamLine_stainlesss
        G4double BeampipeLength = 60.0 * cm;
        G4double Beampipez = flange0z - 0.5 * flange0_thickness + 0.5 * BeampipeLength;
        G4Tubs* Beampipe_tub =
            new G4Tubs("Beampipe_tub", 42.33 * mm, 44.45 * mm, 0.5 * BeampipeLength, 0. * deg, 360. * deg);
        G4LogicalVolume* Beampipe_log = new G4LogicalVolume(Beampipe_tub, mat_stell, "Beampipe_log");
        new G4PVPlacement(0, G4ThreeVector(0, 0, Beampipez), "Beampipe_phys", Beampipe_log, physicalWorld, false, 10,
                          true);

        flange_log0->SetVisAttributes(tube_vis);
        flange_log1->SetVisAttributes(tube_vis);
        Beampipe_log->SetVisAttributes(tube_vis);

        G4double VacuumLength = 70.0 * cm;
        G4double Vacuumz = 0.5 * VacuumLength + Cu0z - 0.5 * Cu0thickness;
        G4Tubs* Vacuum_tub = new G4Tubs("Vacuum_tub", 0, 42.33 * mm, 0.5 * VacuumLength, 0, 360. * deg);
        G4double VacuumCu0z = -0.5 * VacuumLength + 0.5 * Cu0thickness;
        G4Tubs* CuSubstrate_tub1 = new G4Tubs("CuSubstrate_tub1", 11. * mm, 50 * mm, 0.5 * Cu0thickness, 0, 360. * deg);
        G4SubtractionSolid* Vacuum_sub = new G4SubtractionSolid(
            "Vacuum_sub0", Vacuum_tub, CuSubstrate_tub1, G4Transform3D(rot0, G4ThreeVector(0, 0, VacuumCu0z)));  // Cu0
        G4double Vacuumflange0z = VacuumCu0z + 0.5 * Cu0thickness + 0.5 * flange0_thickness;
        Vacuum_sub =
            new G4SubtractionSolid("Vacuumflange0_sub", Vacuum_sub, flange_tub0,
                                   G4Transform3D(rot0, G4ThreeVector(0, 0, Vacuumflange0z)));  // Target flange0
        G4LogicalVolume* Vacuum_log = new G4LogicalVolume(Vacuum_sub, Vacuum, "Vacuum_log");
        new G4PVPlacement(0, G4ThreeVector(0, 0, Vacuumz), "Vacuum_phys", Vacuum_log, physicalWorld, false, 11, true);

        Vacuum_log->SetVisAttributes(G4VisAttributes::GetInvisible);
        // Water_cold
        G4double WaterBack_thickness = 1.3 * mm;
        G4double WaterBackz = -0.5 * Tathickness - 0.5 * WaterBack_thickness;
        G4Tubs* WaterBack_tub =
            new G4Tubs("WaterBack_tub", 0, 12.0 * mm, 0.5 * WaterBack_thickness, 0. * deg, 360. * deg);
        G4LogicalVolume* WaterBack_log = new G4LogicalVolume(WaterBack_tub, mat_Water, "WaterBack_log");
        new G4PVPlacement(0, G4ThreeVector(0, 0, WaterBackz), "WaterBack_phys", WaterBack_log, physicalWorld, false, 12,
                          true);

        WaterBack_log->SetVisAttributes(water_vis);

        // Fe backflange
        G4double Fe0thickness = 1.3 * mm;
        G4double Fe0z = WaterBackz;
        G4Tubs* FeBack_tub0 =
            new G4Tubs("FeBack_tub0", 12.0 * mm, 44.45 * mm, 0.5 * Fe0thickness, 0. * deg, 360. * deg);
        G4LogicalVolume* FeBack_log0 = new G4LogicalVolume(FeBack_tub0, mat_stell, "FeBack_log0");
        new G4PVPlacement(0, G4ThreeVector(0, 0, Fe0z), "FeBack_phys0", FeBack_log0, physicalWorld, false, 13, true);

        G4double Fe1thickness = 6.7 * mm;
        G4double Fe1z = Fe0z - 0.5 * Fe0thickness - 0.5 * Fe1thickness;
        G4Tubs* FeBack_tub1 = new G4Tubs("FeBack_tub1", 0, 44.45 * mm, 0.5 * Fe1thickness, 0. * deg, 360. * deg);
        G4LogicalVolume* FeBack_log1 = new G4LogicalVolume(FeBack_tub1, mat_stell, "FeBack_log1");
        G4PVPlacement* Fe1phys = new G4PVPlacement(0, G4ThreeVector(0, 0, Fe1z), "FeBack_phys1", FeBack_log1,
                                                   physicalWorld, false, 14, true);

        FeBack_log0->SetVisAttributes(tube_vis);
        FeBack_log1->SetVisAttributes(tube_vis);

        // Waterpipe
        G4double Waterpipe1_thickness = Fe1thickness;
        G4Tubs* Waterpipe_tub1 =
            new G4Tubs("Waterpipe_tub1", 0, 1.6 * mm, 0.5 * Waterpipe1_thickness, 0. * deg, 360. * deg);
        G4LogicalVolume* Waterpipe_log1 = new G4LogicalVolume(Waterpipe_tub1, mat_Water, "Waterpipe_log1");
        new G4PVPlacement(0, G4ThreeVector(0, 9.0 * mm, 0), "Waterpipe_phys01", Waterpipe_log1, Fe1phys, false, 17,
                          true);
        new G4PVPlacement(0, G4ThreeVector(0, -9.0 * mm, 0), "Waterpipe_phys02", Waterpipe_log1, Fe1phys, false, 18,
                          true);

        G4double Waterpipe2_thickness = BPE_xyz / 2.0 + 10.0 * cm;
        G4double Waterpipe2z = Fe1z - 0.5 * Fe1thickness - 0.5 * Waterpipe2_thickness;
        G4Tubs* Waterpipe_tub2 =
            new G4Tubs("Waterpipe_tub2", 0, 2.0 * mm, 0.5 * Waterpipe2_thickness, 0. * deg, 360. * deg);
        G4LogicalVolume* Waterpipe_log2 = new G4LogicalVolume(Waterpipe_tub2, mat_Water, "Waterpipe_log2");
        new G4PVPlacement(0, G4ThreeVector(0, 9.0 * mm, Waterpipe2z), "Waterpipe_phys20", Waterpipe_log2, physicalWorld,
                          false, 19, true);
        new G4PVPlacement(0, G4ThreeVector(0, -9.0 * mm, Waterpipe2z), "Waterpipe_phys21", Waterpipe_log2,
                          physicalWorld, false, 20, true);
        Waterpipe_log1->SetVisAttributes(water_vis);
        Waterpipe_log2->SetVisAttributes(water_vis);
    }

    G4SDManager* SDman = G4SDManager::GetSDMpointer();
    GeneHe3detSD* detSD = new GeneHe3detSD("/GeneHe3det/");
    SDman->AddNewDetector(detSD);
    He3active_log->SetSensitiveDetector(detSD);

    return physicalWorld;
}
